import { useRef, useState, useEffect } from "react";
import { MLCEngine } from "@mlc-ai/web-llm";

const MODEL_ID = "Qwen2.5-0.5B-Instruct-q4f16_1-MLC";

const UUID_PROMPT = `Your task is to generate one random UUID version 4 (UUIDv4) and output it. Follow these rules exactly:

1. Output only the UUIDv4 — no explanation, no steps, no formatting, no words.
2. A UUIDv4 is a 36-character string with 5 sections separated by hyphens.
3. The format is: 8-4-4-4-12 hexadecimal characters (0-9, a-f). Example shape:
   xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
   - The 13th character must be "4" (UUIDv4).
   - The 17th character must be one of "8", "9", "a", or "b".

Return only a valid random UUIDv4 string. Do not say anything else.`;

function filterAndFormatUUID(raw: string) {
  let filtered = raw.toLowerCase().replace(/[^0-9a-f-]/g, "");
  filtered = filtered.replace(/-{2,}/g, "-");
  filtered = filtered.replace(/^[-]+|[-]+$/g, "");
  const hex = filtered.replace(/-/g, "");
  let uuid = "";
  const sections = [8, 4, 4, 4, 12];
  let i = 0;
  for (const len of sections) {
    if (hex.length < i + len) break;
    uuid += hex.slice(i, i + len);
    i += len;
    if (uuid.length < 36) uuid += "-";
  }
  uuid = uuid.replace(/-$/, "");
  const isComplete =
    uuid.length === 36 &&
    ["8", "9", "a", "b"].includes(uuid[19]);
  return { uuid, isComplete };
}

export function useUuidGenerator() {
  const engineRef = useRef<MLCEngine | null>(null);
  const abortRef = useRef<AbortController | null>(null);
  const [output, setOutput] = useState("");
  const [loading, setLoading] = useState(false);
  const [progress, setProgress] = useState<string | null>(null);
  const [error, setError] = useState<string | null>(null);

  const loadModel = async () => {
    setProgress("Loading model...");
    setError(null);
    if (!engineRef.current) {
      engineRef.current = null;
    }
    engineRef.current = new MLCEngine({
      initProgressCallback: (p: any) => {
        if (p && typeof p === "object" && "progress" in p) {
          setProgress(
            `Downloading model: ${Math.round((p.progress || 0) * 100)}%`
          );
        } else {
          setProgress("Initializing model...");
        }
      },
    });
    try {
      await engineRef.current.reload(MODEL_ID);
      setProgress(null);
    } catch (e: any) {
      setError("Model load failed: " + (e?.message || e.toString()));
      setProgress(null);
      throw e;
    }
  };

  const handleGenerate = async () => {
    setError(null);
    setOutput("");
    setLoading(true);
    setProgress(null);
    if (abortRef.current) abortRef.current.abort();
    abortRef.current = new AbortController();
    try {
      await loadModel();
      const engine = engineRef.current!;
      const messages = [
        {
          role: "user",
          content: UUID_PROMPT,
        },
      ];
      let buffer = "";
      let done = false;
      const chunks = await engine.chat.completions.create({
        messages: messages as any,
        temperature: 1,
        stream: true,
        stream_options: { include_usage: false },
      });
      for await (const chunk of chunks) {
        if (done) break;
        const delta = chunk.choices[0]?.delta.content || "";
        buffer += delta;
        const { uuid, isComplete } = filterAndFormatUUID(buffer);
        setOutput(uuid);
        if (isComplete) {
          abortRef.current.abort();
          done = true;
          setLoading(false);
          setProgress(null);
        }
      }
    } catch (e: any) {
      if (abortRef.current) abortRef.current.abort();
      setLoading(false);
      setProgress(null);
      if (e?.name === "AbortError") {
        // Ignore abort
      } else {
        setError("Generation failed: " + (e?.message || e.toString()));
      }
    } finally {
      if (abortRef.current) abortRef.current.abort();
      setLoading(false);
      setProgress(null);
    }
    if (abortRef.current) abortRef.current.abort();
    setLoading(false);
    setProgress(null);
  };

  const handleAbort = () => {
    if (abortRef.current) abortRef.current.abort();
    setLoading(false);
    setProgress(null);
  };

  useEffect(() => {
    return () => {
      if (abortRef.current) abortRef.current.abort();
    };
  }, []);

  return {
    output,
    loading,
    progress,
    error,
    handleGenerate,
    handleAbort,
  };
} 